/****************************************************************************

 KHotKeys
 
 Copyright (C) 1999-2001 Lubos Lunak <l.lunak@kde.org>

 Distributed under the terms of the GNU General Public License version 2.
 
 $Id: triggers_tab.cpp,v 1.19 2001/11/20 14:00:11 seli Exp $

****************************************************************************/

#define __TRIGGERS_TAB_CPP

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "triggers_tab.h"

#include <assert.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qpopupmenu.h>

#include <kdebug.h>
#include <klocale.h>
#include <kapp.h>

#include "kcmkhotkeys.h"
#include "gesturerecordpage.h"
#include "windowdef_list_widget.h"
#include "window_trigger_widget.h"

namespace KHotKeys
{

// Triggers_tab

Triggers_tab::Triggers_tab( QWidget* parent_P, const char* name_P )
    : Triggers_tab_ui( parent_P, name_P ), selected_item( NULL )
    {
    QPopupMenu* popup = new QPopupMenu; // CHECKME looks like setting parent doesn't work
    popup->insertItem( i18n( "Single shortcut trigger" ), TYPE_SINGLE_SHORTCUT_TRIGGER );
    popup->insertItem( i18n( "Window trigger" ), TYPE_WINDOW_TRIGGER );
    popup->insertItem( i18n( "Gesture trigger" ), TYPE_GESTURE_TRIGGER );
    connect( popup, SIGNAL( activated( int )), SLOT( new_selected( int )));
    new_button->setPopup( popup );
    copy_button->setEnabled( false );    
    modify_button->setEnabled( false );    
    delete_button->setEnabled( false );    
    clear_data();
    // KHotKeys::Module::changed()
    connect( new_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( copy_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( modify_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( delete_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( comment_lineedit, SIGNAL( textChanged( const QString& )),
        module, SLOT( changed()));
    }

Triggers_tab::~Triggers_tab()
    {
    delete new_button->popup(); // CHECKME
    }
        
void Triggers_tab::clear_data()
    {
    comment_lineedit->setText( "" );
    triggers_listbox->clear();
    }
    
void Triggers_tab::set_data( const Trigger_list* data_P )
    {
    comment_lineedit->setText( data_P->comment());
    Trigger_list_item* after = NULL;
    triggers_listbox->clear();
    for( Trigger_list::Iterator it( *data_P );
         *it;
         ++it )
        after = create_listbox_item( *it, triggers_listbox, after, true );
    }
    
Trigger_list* Triggers_tab::get_data( Action_data* data_P ) const
    {
    Trigger_list* list = new Trigger_list( comment_lineedit->text());
    for( QListBoxItem* pos = triggers_listbox->firstItem();
         pos != NULL;
         pos = pos->next())
        list->append( static_cast< Trigger_list_item* >( pos )->trigger()->copy( data_P ));
    return list;
    }

void Triggers_tab::new_selected( int type_P )
    {
        // MEMORY LEAK!
        // There is a memory leak here. A new trigger is created and
        // given to the dialog. If the dialog is canceled, the new trigger
        // is never deleted! -mike pilone
        
    Trigger_dialog* dlg = NULL;
    switch( type_P )
        {
        case TYPE_SINGLE_SHORTCUT_TRIGGER: // Single_shortcut_trigger
            dlg = new Single_shortcut_trigger_dialog(
                new Single_shortcut_trigger( NULL, 0 )); // CHECKME NULL ?
          break;
        case TYPE_WINDOW_TRIGGER: // Window trigger
            dlg = new Window_trigger_dialog( new Window_trigger( NULL, new Windowdef_list( "" ),
                0 )); // CHECKME NULL ?
          break;
        case TYPE_GESTURE_TRIGGER: // Gesture trigger
            dlg = new Gesture_trigger_dialog(new Gesture_trigger(NULL
                                             , QString::null), this);
            break;
        }
    if( dlg != NULL )
        {
        Trigger* trg = dlg->edit_trigger();
        if( trg != NULL )
            triggers_listbox->setSelected( create_listbox_item( trg, triggers_listbox,
                selected_item, false ), true );
        delete dlg;
        }
    }
    
void Triggers_tab::copy_pressed()
    {
    triggers_listbox->setSelected( create_listbox_item( selected_item->trigger(),
        triggers_listbox, selected_item, true ), true );
    }
    
void Triggers_tab::delete_pressed()
    {
    delete selected_item; // CHECKME snad vyvola signaly pro enable()
    }
    
void Triggers_tab::modify_pressed()
    {
    edit_listbox_item( selected_item );
    }

void Triggers_tab::current_changed( QListBoxItem* item_P )
    {
    if( item_P == selected_item )
        return;
    selected_item = static_cast< Trigger_list_item* >( item_P );
    triggers_listbox->setSelected( item_P, true );
    copy_button->setEnabled( item_P != NULL );
    modify_button->setEnabled( item_P != NULL );
    delete_button->setEnabled( item_P != NULL );
    }

Trigger_list_item* Triggers_tab::create_listbox_item( Trigger* trigger_P,
    QListBox* parent_P, QListBoxItem* after_P, bool copy_P )
    {
    Trigger* new_trg = copy_P ? trigger_P->copy( NULL ) : trigger_P; // CHECKME NULL ?
    if( after_P == NULL )
        return new Trigger_list_item( parent_P, new_trg );
    else
        return new Trigger_list_item( parent_P, after_P, new_trg );
    }

void Triggers_tab::edit_listbox_item( Trigger_list_item* item_P )
    {
    Trigger_dialog* dlg = NULL;
    if( Single_shortcut_trigger* trg = dynamic_cast< Single_shortcut_trigger* >( item_P->trigger()))
        dlg = new Single_shortcut_trigger_dialog( trg );
    else if( Window_trigger* trg = dynamic_cast< Window_trigger* >( item_P->trigger()))
        dlg = new Window_trigger_dialog( trg );
     else if( Gesture_trigger* trg = dynamic_cast< Gesture_trigger* >( item_P->trigger()))
        dlg = new Gesture_trigger_dialog( trg, this );
// CHECKME TODO dalsi
    else
        assert( false );
    Trigger* new_trigger = dlg->edit_trigger();
    if( new_trigger != NULL )
        item_P->set_trigger( new_trigger );
    delete dlg;
    }
    
// Trigger_list_item

QString Trigger_list_item::text() const
    {
    return trigger()->description();
    }
    
// Single_shortcut_trigger_widget

Single_shortcut_trigger_widget::Single_shortcut_trigger_widget( QWidget* parent_P, const char* )
#if KDE_VERSION < 220 // CHECKME
    : KKeyChooser( init_map(), parent_P, false )
#else
    : KKeyChooser( init_map(), parent_P, false, true, true )
#endif
    {
    clear_data();
    // KHotKeys::Module::changed()
    connect( this, SIGNAL( keyChange()), module, SLOT( changed()));
    }

// ugly hack to avoid the braindamaged way KKeyChooser constructor is written
KKeyEntryMap* Single_shortcut_trigger_widget::init_map()
    {
    map = new KKeyEntryMap;
    KKeyEntry entry;
    entry.aCurrentKeyCode = entry.aConfigKeyCode = 0;
    entry.bConfigurable = true;
    entry.bEnabled = true;
    entry.descr = i18n( "Select keyboard shortcut" ); // CHECKME nebo nazev akce ?
    map->insert( "key", entry );
    return map;
    }

Single_shortcut_trigger_widget::~Single_shortcut_trigger_widget()
    {
    delete map;
    }

void Single_shortcut_trigger_widget::clear_data()
    {
    ( *map )[ "key" ].aConfigKeyCode = 0;
    listSync();
    }    
    
void Single_shortcut_trigger_widget::set_data( const Single_shortcut_trigger* data_P )
    {
    ( *map )[ "key" ].aConfigKeyCode = data_P->keycode();
    listSync();
    }

Single_shortcut_trigger* Single_shortcut_trigger_widget::get_data( Action_data* data_P ) const
    {
    return new Single_shortcut_trigger( data_P, ( *map )[ "key" ].aConfigKeyCode );
    }

// Single_shortcut_trigger_dialog

Single_shortcut_trigger_dialog::Single_shortcut_trigger_dialog( Single_shortcut_trigger* trigger_P )
    : KDialogBase( NULL, NULL, true, "", Ok | Cancel ), // CHECKME caption
        trigger( NULL )
    {
    widget = new Single_shortcut_trigger_widget( this );
    widget->set_data( trigger_P );
    setMainWidget( widget );
    }

Trigger* Single_shortcut_trigger_dialog::edit_trigger()
    {
    exec();
    return trigger;
    }
    
void Single_shortcut_trigger_dialog::accept()
    {
    KDialogBase::accept();
    trigger = widget->get_data( NULL ); // CHECKME NULL ?
    }
    
// Window_trigger_dialog

Window_trigger_dialog::Window_trigger_dialog( Window_trigger* trigger_P )
    : KDialogBase( NULL, NULL, true, "", Ok | Cancel ), // CHECKME caption
        trigger( NULL )
    {
    widget = new Window_trigger_widget( this );
    widget->set_data( trigger_P );
    setMainWidget( widget );
    }

Trigger* Window_trigger_dialog::edit_trigger()
    {
    exec();
    return trigger;
    }
    
void Window_trigger_dialog::accept()
    {
    KDialogBase::accept();
    trigger = widget->get_data( NULL ); // CHECKME NULL ?
    }
    
// Window_trigger_dialog

Gesture_trigger_dialog::Gesture_trigger_dialog( Gesture_trigger* trigger_P,
                                                 QWidget *parent)
    : KDialogBase( parent, NULL, true, "", Ok | Cancel ), // CHECKME caption
        _trigger( trigger_P ), _page(NULL)
    {
    _page = new GestureRecordPage(_trigger->gesturecode(),
                                  this, "GestureRecordPage");
   
    connect(_page, SIGNAL(readyForNext(bool)),
            this, SLOT(enableButtonOK(bool)));
   
    setMainWidget( _page );
    }

Trigger* Gesture_trigger_dialog::edit_trigger()
    {
    if (exec())
    {
        _trigger->setGesturecode(_page->getGesture());
        return _trigger;
    }
    else
    {
        return NULL;
    }
    }
    
} // namespace KHotKeys

#include "triggers_tab.moc"
