/****************************************************************************

 KHotKeys
 
 Copyright (C) 1999-2001 Lubos Lunak <l.lunak@kde.org>

 Distributed under the terms of the GNU General Public License version 2.
 
 $Id: triggers_tab.h,v 1.14 2001/11/20 14:00:11 seli Exp $

****************************************************************************/

#ifndef __TRIGGERS_TAB_H
#define __TRIGGERS_TAB_H

#include <qlistbox.h>

#include <kkeydialog.h>
#include <kdialogbase.h>

#include <triggers.h>
#include <triggers_tab_ui.h>
#include <window_trigger_widget.h>

namespace KHotKeys
{

class Windowdef_list;
class Action_data;
class Trigger_list_item;

class Triggers_tab
    : public Triggers_tab_ui
    {
    Q_OBJECT
    public:
        Triggers_tab( QWidget* parent_P = NULL, const char* name_P = NULL );
        virtual ~Triggers_tab();
        void set_data( const Trigger_list* data_P );
        Trigger_list* get_data( Action_data* data_P ) const;
    public slots:
        void clear_data();
    protected:
        Trigger_list_item* create_listbox_item( Trigger* trigger_P, QListBox* parent_P,
            QListBoxItem* after_P, bool copy_P );
        void edit_listbox_item( Trigger_list_item* item_P );
        Trigger_list_item* selected_item;
        enum type_t { TYPE_SINGLE_SHORTCUT_TRIGGER, TYPE_WINDOW_TRIGGER,
                      TYPE_GESTURE_TRIGGER };
    protected slots:
        void new_selected( int type_P );
        virtual void copy_pressed();
        virtual void delete_pressed();
        virtual void modify_pressed();
        virtual void current_changed( QListBoxItem* item_P );
    };

class Trigger_list_item
    : public QListBoxText
    {
    public:
        Trigger_list_item( QListBox* parent_P, Trigger* trigger_P );
        Trigger_list_item( QListBox* parent_P, QListBoxItem* after_P, Trigger* trigger_P );
        virtual ~Trigger_list_item();
        virtual QString text() const;
        Trigger* trigger() const;
        void set_trigger( Trigger* trigger_P );
    protected:
        Trigger* _trigger; // owns
    };
        
class Trigger_dialog
    {
    public:
        virtual Trigger* edit_trigger() = 0;
        virtual ~Trigger_dialog();
    };
    
class Single_shortcut_trigger_widget
    : public KKeyChooser
    {
    Q_OBJECT
    public:
        Single_shortcut_trigger_widget( QWidget* parent_P = NULL, const char* name_P = NULL );
        virtual ~Single_shortcut_trigger_widget();
        void set_data( const Single_shortcut_trigger* trigger_P );
        Single_shortcut_trigger* get_data( Action_data* data_P ) const;
    public slots:
        void clear_data();
    private:
        KKeyEntryMap* init_map();
        KKeyEntryMap* map;
    };
    
typedef Single_shortcut_trigger_widget Single_shortcut_trigger_tab;
    
class Single_shortcut_trigger_dialog
    : public KDialogBase, public Trigger_dialog
    {
    Q_OBJECT
    public:
        Single_shortcut_trigger_dialog( Single_shortcut_trigger* trigger_P );
        virtual Trigger* edit_trigger();
    protected:
        virtual void accept();
        Single_shortcut_trigger_widget* widget;
        Single_shortcut_trigger* trigger;
    };
    
class Window_trigger_dialog
    : public KDialogBase, public Trigger_dialog
    {
    Q_OBJECT
    public:
        Window_trigger_dialog( Window_trigger* trigger_P );
        virtual Trigger* edit_trigger();
    protected:
        virtual void accept();
        Window_trigger_widget* widget;
        Window_trigger* trigger;
    };

class GestureRecordPage;
class Gesture_trigger_dialog
    : public KDialogBase, public Trigger_dialog
    {
    Q_OBJECT
    public:
        Gesture_trigger_dialog( Gesture_trigger* trigger_P, QWidget *parent );
        virtual Trigger* edit_trigger();
       
    private:
        Gesture_trigger* _trigger;
        GestureRecordPage *_page;
    };        
            
//***************************************************************************
// Inline
//***************************************************************************

// Trigger_list_item

inline
Trigger_list_item::Trigger_list_item( QListBox* parent_P, Trigger* trigger_P )
    : QListBoxText( parent_P, trigger_P->description()), _trigger( trigger_P )
    {
    }
    
inline
Trigger_list_item::Trigger_list_item( QListBox* parent_P, QListBoxItem* after_P,
    Trigger* trigger_P )
    : QListBoxText( parent_P, trigger_P->description(), after_P ), _trigger( trigger_P )
    {
    }
    
inline
Trigger_list_item::~Trigger_list_item()
    {
    delete _trigger;
    }

inline
Trigger* Trigger_list_item::trigger() const
    {
    return _trigger;
    }
    
inline
void Trigger_list_item::set_trigger( Trigger* trigger_P )
    {
    // Needed since Gesture trigger returns the same trigger, just
    // updated
    if (_trigger != trigger_P)
    {
        delete _trigger;
        _trigger = trigger_P;
    }
    }

// Trigger_dialog

inline
Trigger_dialog::~Trigger_dialog()
    {
    }
    
} // namespace KHotKeys

#endif
