#include <assert.h>
#include <stdio.h>

#include <kapp.h>

#include "gesture.h"
#include "khotkeysglobal.h"

#include <X11/X.h>
#include <X11/Xlib.h>

#define GESTURE_POLL_TIME 5
#define GESTURE_MOUSE_SLEEP_PERIOD 30

namespace KHotKeys {

//////////////////////////////
// Gesture
Gesture::Gesture( bool enable_signals_P, QObject* parent_P )
    : QObject(parent_P), _enable_signals(enable_signals_P),
    _display(NULL), _xCoord(0), _yCoord(0),
    _lastGridCoord(0, 0), _mousePaused(0),
    _numStrokesRecorded(0), _lifeLength(3),
    _mousePollTimer(this, "mousePollTimer")
  {
      assert(gesture_handler == NULL);
      gesture_handler = this;
   
     // Connect to the X server - needed for mouse events
      initXCon();
   
     connect(&_mousePollTimer, SIGNAL(timeout()),
	         this, SLOT(slotMousePollTimeout()));
      
     if (_enable_signals)
         _mousePollTimer.start(GESTURE_POLL_TIME, false);
      
       fprintf(stderr, "Gesture::Gesture\n");
  }
  
Gesture::~Gesture()
    {
    }
    

void Gesture::initXCon()
  {
  _display = KApplication::kApplication()->getDisplay();

  _root = RootWindow(_display,DefaultScreen(_display));
  }
  
  
/////////////////////////////
// SLOTS

void Gesture::slotMousePollTimeout()
  {
  XQueryPointer (_display, _root, &_rootIDRet, &_childIDRet, &_xCoord,
		 &_yCoord, &_winX, &_winY, &_stateMask);

  QPoint gridCoord(_xCoord, _yCoord);

  //kdDebug() << "Num points recorded: " << _gridPointList.count() << "\n";
  
  if (_lastGridCoord == gridCoord)
    {
    if ((_mousePaused > GESTURE_MOUSE_SLEEP_PERIOD) 
        && (_numStrokesRecorded > 1))
      {
      // process list
      if (stroke_trans (_sequence) == TRUE)
	     {
         QString gesturecode(_sequence);
         qDebug("Got gesture: %s", _sequence);
         if ((gesturecode != QString::null) && (_enable_signals))
             emit handle_gesture(gesturecode);
	     }

      // Add this point for the next translation
      stroke_record(gridCoord.x(), gridCoord.y());
      _numStrokesRecorded = 1;
      }

      _mousePaused++;

      // sanity check
      if (_mousePaused > 10000)
          _mousePaused = GESTURE_MOUSE_SLEEP_PERIOD+1;
    }
  else
    {
    _mousePaused = 0;

    stroke_record(gridCoord.x(), gridCoord.y());
    _numStrokesRecorded++;
    }

  _lastGridCoord = gridCoord;
  }
    
} // namespace KHotKeys

#include "gesture.moc"

